"use strict";
/**
 * Core Memory Engine - Persistent AI Assistant Memory
 */
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || (function () {
    var ownKeys = function(o) {
        ownKeys = Object.getOwnPropertyNames || function (o) {
            var ar = [];
            for (var k in o) if (Object.prototype.hasOwnProperty.call(o, k)) ar[ar.length] = k;
            return ar;
        };
        return ownKeys(o);
    };
    return function (mod) {
        if (mod && mod.__esModule) return mod;
        var result = {};
        if (mod != null) for (var k = ownKeys(mod), i = 0; i < k.length; i++) if (k[i] !== "default") __createBinding(result, mod, k[i]);
        __setModuleDefault(result, mod);
        return result;
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
exports.MemoryEngine = void 0;
const path = __importStar(require("path"));
const fs = __importStar(require("fs-extra"));
const sqlite3_1 = require("sqlite3");
const uuid_1 = require("uuid");
class MemoryEngine {
    constructor(projectPath) {
        this.db = null;
        this.config = {
            projectPath: projectPath || process.cwd(),
            dbPath: path.join(projectPath || process.cwd(), '.codecontext', 'memory.db'),
            scanIgnore: [
                'node_modules',
                '.git',
                'dist',
                'build',
                '.next',
                '.nuxt',
                'target',
                '__pycache__',
                '.venv',
                'venv',
                '.codecontext'
            ],
            autoScan: true,
            maxMemories: 10000,
            retentionDays: 365
        };
    }
    async initProject(projectPath, force = false) {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        // Create .codecontext directory
        const codecontextDir = path.join(projectPath, '.codecontext');
        await fs.ensureDir(codecontextDir);
        // Check if already initialized
        if (!force && await fs.pathExists(this.config.dbPath)) {
            throw new Error('Project already initialized. Use --force to reinitialize.');
        }
        // Initialize database
        await this.initDatabase();
        // Create project record
        const projectName = path.basename(projectPath);
        const projectId = (0, uuid_1.v4)();
        await this.query(`
      INSERT INTO projects (id, name, path, created_at, last_active)
      VALUES (?, ?, ?, ?, ?)
    `, [projectId, projectName, projectPath, new Date().toISOString(), new Date().toISOString()]);
        // Create config file
        const configPath = path.join(codecontextDir, 'config.json');
        await fs.writeJson(configPath, {
            projectId,
            projectName,
            version: '1.0.0',
            initialized: new Date().toISOString()
        }, { spaces: 2 });
        // Create .gitignore entry
        const gitignorePath = path.join(projectPath, '.gitignore');
        if (await fs.pathExists(gitignorePath)) {
            const gitignore = await fs.readFile(gitignorePath, 'utf8');
            if (!gitignore.includes('.codecontext')) {
                await fs.appendFile(gitignorePath, '\n# CodeContext Memory\n.codecontext/\n');
            }
        }
        await this.logActivity('init', 'Project initialized with memory capabilities');
    }
    async getProjectStatus(projectPath) {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        if (!await fs.pathExists(this.config.dbPath)) {
            throw new Error('Project not initialized. Run "codecontext init" first.');
        }
        await this.connectDatabase();
        const project = await this.queryOne(`
      SELECT * FROM projects WHERE path = ?
    `, [projectPath]);
        if (!project) {
            throw new Error('Project not found in memory database');
        }
        const [filesCount, conversationsCount, patternsCount] = await Promise.all([
            this.queryOne('SELECT COUNT(*) as count FROM files'),
            this.queryOne('SELECT COUNT(*) as count FROM memories WHERE type = "conversation"'),
            this.queryOne('SELECT COUNT(*) as count FROM patterns')
        ]);
        const recentActivity = await this.query(`
      SELECT * FROM activities 
      ORDER BY created_at DESC 
      LIMIT 10
    `);
        // Get database size
        const stats = await fs.stat(this.config.dbPath);
        const memorySize = this.formatBytes(stats.size);
        return {
            projectId: project.id,
            projectName: project.name,
            createdAt: new Date(project.created_at),
            lastActive: new Date(project.last_active),
            filesTracked: filesCount.count,
            conversations: conversationsCount.count,
            patterns: patternsCount.count,
            memorySize,
            recentActivity: recentActivity.map((a) => ({
                timestamp: new Date(a.created_at),
                description: a.description,
                type: a.type
            }))
        };
    }
    async storeProjectAnalysis(projectPath, analysis) {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        await this.connectDatabase();
        // Store files
        for (const file of analysis.files) {
            await this.query(`
        INSERT OR REPLACE INTO files (path, language, size, lines, last_modified, hash)
        VALUES (?, ?, ?, ?, ?, ?)
      `, [file.path, file.language, file.size, file.lines, file.lastModified.toISOString(), file.hash]);
        }
        // Store patterns
        for (const pattern of analysis.patterns) {
            await this.query(`
        INSERT OR REPLACE INTO patterns (id, type, name, description, frequency, confidence, examples, file, line_start, line_end)
        VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?)
      `, [
                pattern.id,
                pattern.type,
                pattern.name,
                pattern.description,
                pattern.frequency,
                pattern.confidence,
                JSON.stringify(pattern.examples),
                pattern.file,
                pattern.lines[0],
                pattern.lines[1]
            ]);
        }
        // Update project metrics
        await this.query(`
      UPDATE projects SET 
        last_active = ?,
        total_files = ?,
        total_lines = ?,
        complexity = ?
      WHERE path = ?
    `, [
            new Date().toISOString(),
            analysis.metrics.totalFiles,
            analysis.metrics.totalLines,
            analysis.metrics.complexity,
            projectPath
        ]);
        await this.logActivity('scan', `Analyzed ${analysis.files.length} files, found ${analysis.patterns.length} patterns`);
    }
    async storeMemory(projectPath, memory) {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        await this.connectDatabase();
        const id = (0, uuid_1.v4)();
        await this.query(`
      INSERT INTO memories (id, type, content, context, created_at, tags, metadata)
      VALUES (?, ?, ?, ?, ?, ?, ?)
    `, [
            id,
            memory.type,
            memory.content,
            memory.context || null,
            memory.timestamp.toISOString(),
            JSON.stringify(memory.tags || []),
            JSON.stringify(memory.metadata || {})
        ]);
        await this.logActivity('memory', `Stored ${memory.type}: ${memory.content.substring(0, 50)}...`);
        return id;
    }
    async searchMemories(projectPath, query, options = {}) {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        await this.connectDatabase();
        let sql = 'SELECT * FROM memories WHERE 1=1';
        const params = [];
        if (options.type) {
            sql += ' AND type = ?';
            params.push(options.type);
        }
        if (query) {
            sql += ' AND (content LIKE ? OR context LIKE ?)';
            params.push(`%${query}%`, `%${query}%`);
        }
        if (options.since) {
            sql += ' AND created_at >= ?';
            params.push(options.since.toISOString());
        }
        sql += ' ORDER BY created_at DESC';
        if (options.limit) {
            sql += ' LIMIT ?';
            params.push(options.limit);
        }
        const results = await this.query(sql, params);
        return results.map((row) => ({
            id: row.id,
            type: row.type,
            content: row.content,
            context: row.context,
            timestamp: new Date(row.created_at),
            tags: JSON.parse(row.tags || '[]'),
            metadata: JSON.parse(row.metadata || '{}')
        }));
    }
    async exportMemory(projectPath, format = 'json') {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        await this.connectDatabase();
        const status = await this.getProjectStatus(projectPath);
        const memories = await this.searchMemories(projectPath);
        const patterns = await this.query('SELECT * FROM patterns');
        const files = await this.query('SELECT * FROM files');
        const data = {
            project: status,
            memories,
            patterns,
            files,
            exportedAt: new Date().toISOString()
        };
        if (format === 'json') {
            return JSON.stringify(data, null, 2);
        }
        else {
            // Markdown format
            let md = `# CodeContext Memory Export\n\n`;
            md += `**Project:** ${status.projectName}\n`;
            md += `**Exported:** ${new Date().toLocaleString()}\n\n`;
            md += `## Statistics\n\n`;
            md += `- Files Tracked: ${status.filesTracked}\n`;
            md += `- Conversations: ${status.conversations}\n`;
            md += `- Patterns: ${status.patterns}\n`;
            md += `- Memory Size: ${status.memorySize}\n\n`;
            if (memories.length > 0) {
                md += `## Memories\n\n`;
                memories.forEach(memory => {
                    md += `### ${memory.type} - ${memory.timestamp.toLocaleDateString()}\n\n`;
                    md += `${memory.content}\n\n`;
                    if (memory.context) {
                        md += `*Context: ${memory.context}*\n\n`;
                    }
                });
            }
            return md;
        }
    }
    async clearMemory(projectPath) {
        this.config.projectPath = projectPath;
        this.config.dbPath = path.join(projectPath, '.codecontext', 'memory.db');
        await this.connectDatabase();
        await this.query('DELETE FROM memories');
        await this.query('DELETE FROM patterns');
        await this.query('DELETE FROM files');
        await this.query('DELETE FROM activities');
        await this.logActivity('clear', 'All memory data cleared');
    }
    async initDatabase() {
        await this.connectDatabase();
        const schema = `
      CREATE TABLE IF NOT EXISTS projects (
        id TEXT PRIMARY KEY,
        name TEXT NOT NULL,
        path TEXT NOT NULL UNIQUE,
        created_at TEXT NOT NULL,
        last_active TEXT NOT NULL,
        total_files INTEGER DEFAULT 0,
        total_lines INTEGER DEFAULT 0,
        complexity TEXT DEFAULT 'unknown'
      );

      CREATE TABLE IF NOT EXISTS files (
        path TEXT PRIMARY KEY,
        language TEXT,
        size INTEGER,
        lines INTEGER,
        last_modified TEXT,
        hash TEXT
      );

      CREATE TABLE IF NOT EXISTS patterns (
        id TEXT PRIMARY KEY,
        type TEXT NOT NULL,
        name TEXT NOT NULL,
        description TEXT,
        frequency INTEGER DEFAULT 1,
        confidence REAL DEFAULT 0.0,
        examples TEXT,
        file TEXT,
        line_start INTEGER,
        line_end INTEGER
      );

      CREATE TABLE IF NOT EXISTS memories (
        id TEXT PRIMARY KEY,
        type TEXT NOT NULL,
        content TEXT NOT NULL,
        context TEXT,
        created_at TEXT NOT NULL,
        tags TEXT DEFAULT '[]',
        metadata TEXT DEFAULT '{}'
      );

      CREATE TABLE IF NOT EXISTS activities (
        id INTEGER PRIMARY KEY AUTOINCREMENT,
        type TEXT NOT NULL,
        description TEXT NOT NULL,
        created_at TEXT NOT NULL
      );

      CREATE INDEX IF NOT EXISTS idx_memories_type ON memories(type);
      CREATE INDEX IF NOT EXISTS idx_memories_created_at ON memories(created_at);
      CREATE INDEX IF NOT EXISTS idx_patterns_type ON patterns(type);
      CREATE INDEX IF NOT EXISTS idx_files_language ON files(language);
    `;
        await this.execute(schema);
    }
    async connectDatabase() {
        if (this.db)
            return;
        return new Promise((resolve, reject) => {
            this.db = new sqlite3_1.Database(this.config.dbPath, (err) => {
                if (err)
                    reject(err);
                else
                    resolve();
            });
        });
    }
    async query(sql, params = []) {
        if (!this.db)
            throw new Error('Database not connected');
        return new Promise((resolve, reject) => {
            this.db.all(sql, params, (err, rows) => {
                if (err)
                    reject(err);
                else
                    resolve(rows);
            });
        });
    }
    async queryOne(sql, params = []) {
        const results = await this.query(sql, params);
        return results[0];
    }
    async execute(sql, params = []) {
        if (!this.db)
            throw new Error('Database not connected');
        return new Promise((resolve, reject) => {
            this.db.exec(sql, (err) => {
                if (err)
                    reject(err);
                else
                    resolve();
            });
        });
    }
    async logActivity(type, description) {
        await this.query(`
      INSERT INTO activities (type, description, created_at)
      VALUES (?, ?, ?)
    `, [type, description, new Date().toISOString()]);
    }
    formatBytes(bytes) {
        if (bytes === 0)
            return '0 B';
        const k = 1024;
        const sizes = ['B', 'KB', 'MB', 'GB'];
        const i = Math.floor(Math.log(bytes) / Math.log(k));
        return parseFloat((bytes / Math.pow(k, i)).toFixed(2)) + ' ' + sizes[i];
    }
}
exports.MemoryEngine = MemoryEngine;
//# sourceMappingURL=MemoryEngine.js.map